package apps;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class MBDGContextExpander {
	
	private static final String MBDG_REPLACEABLE_CONTEXT_ID = "MBDGReplaceableContextId";
	private static final String MBDG_REPLACEABLE_CONTAINER = "MBDGReplaceableContainer";
	private static final String REPLACEABLE_QUERIES = "replaceableQueries";
	private static final String MBDG_CONTEXT = "MBDGContext";
	private StringBuilder template;

	public MBDGContextExpander(StringBuilder template) {
		this.template = template;
	}
	
	public void expandReplaceableQueries(MBDGElement element) {
		if (!element.isRoot()) {
			String queries = MBDGUtils.getTagValue(element.getElement(), REPLACEABLE_QUERIES);
			if (queries != null && !queries.equals("")) {
				MBDGExpander expander = new MBDGExpander(template);
				String[] pairs = queries.split(";");
				for (String pair : pairs) {
					String[] tokens = pair.split(",");
					if (tokens.length == 2) {
						String query1 = tokens[0];
						String query2 = tokens[1];
						if (!query2.startsWith("${MBDG_")) {
							expander.expand(query1, query2);
						}
					}
				}
			}
		}
	}		

	public void collectContextHandleAndQuery(MBDGElement element) {
		StringBuilder contextContent = getElementSectionByName(element, MBDG_CONTEXT);
		if (contextContent.length() > 0) {
			String data = getFirstDataSection(contextContent);
			String handle = extractFromDataSection(data, "handle=\"", "\"");
			element.setElementHandle(handle);
			String context = extractFromDataSection(data, "context=\"", "\"");
			element.setElementContext(context);			
			String query = extractFromDataSection(data, "<query>", "</query>");
			element.setElementQuery(query);
			
			//collectReplaceableContextHandle(element, contextContent, context);
		}
	}

	protected void collectReplaceableContextHandle(MBDGElement element, StringBuilder contextContent, String context) {
		String replaceableContext = context;
		if (contextContent.indexOf(MBDG_REPLACEABLE_CONTEXT_ID) == -1) {
			StringBuilder replaceableContextContent = getElementSectionByName(element, MBDG_REPLACEABLE_CONTEXT_ID);
			if (replaceableContextContent.length() > 0) {
				String data = getFirstDataSection(replaceableContextContent);
				replaceableContext = extractFromDataSection(data, "context=\"", "\"");					
			}
		}
		element.setReplaceableContext(replaceableContext);		
	}
	
	protected void removeReplaceableContainerAndUpdateContext(MBDGElement element) {
		if (!element.isRoot()) {
			String ownerHandle = element.getOwnerElementHandle();
			if (!ownerHandle.equals("")) {
				StringBuilder content = getElementSectionByName(element, MBDG_REPLACEABLE_CONTAINER);
				if (content.length() > 0) {
					removeReplaceableContainer(content);
					String replaceableContext = element.getElementContext();
					if (!replaceableContext.equals("")) {
						boolean expanded = replaceContextHandle(replaceableContext, ownerHandle);
						if (expanded) {
							element.setElementContext(ownerHandle);
						}
					}
				}
			}
		}
	}
	
	protected boolean replaceContextHandle(String replaceableContext, String ownerHandle) {
		String pattern1 = String.format("context=\"%s\"", replaceableContext);
		String pattern2 = String.format("context=\"%s\"", ownerHandle);
		return new MBDGExpander(template).expand(pattern1, pattern2);
	}

	protected StringBuilder getElementSectionByName(MBDGElement element, String sectionName) {
		final String openTag = "<element ";
		final String closeTag = "</element>";
		final String pattern = String.format("<property name=\"name\" value=\"\\s*%s\\s*\"></property>", sectionName);
		
		return getSection(openTag, closeTag, pattern);
	}
	
	protected void removeReplaceableContainer(StringBuilder content) {
		final String openTag = "<element ";
		final String closeTag = "</element>";

		int start = content.indexOf(openTag,1);
		if (start > 0) {
			int end = content.lastIndexOf(closeTag, content.length()-closeTag.length()-1);
			if (end >= start) {		
				StringBuilder subContent = new StringBuilder(content.substring(start, end+closeTag.length()));
				template.delete(0, template.length());
				template.append(subContent);
			}
		}
	}	

	protected StringBuilder getSection(String openTag, String closeTag, String str_pattern) {
		StringBuilder section = new StringBuilder();

		int start = 0;
		Pattern pattern = Pattern.compile(str_pattern);
		Matcher matcher = pattern.matcher(template);
		if (matcher.find()) {
			start = matcher.start();
		}
		
		if (start > 0) {
			int end = template.indexOf(closeTag, start);
			if (end >= 0) {
				int count = countTags(closeTag, end);
				start = findTagPos(openTag, count+1);
				if (start >= 0 && end >= start) {
					section.append(template.substring(start, end+closeTag.length()));
				}
			}
		}
		return section;
	}
	
	private int findTagPos(String pattern, int tagsCount) {
		int pos = -1, tmpPos = -1;
		int count = 0;
		while ( count < tagsCount) {
			tmpPos = template.indexOf(pattern, tmpPos+1);
			if (tmpPos >= 0) {
				pos = tmpPos;
				count++;
			} else {
				break;
			}
		}
		return pos;
	}

	private int countTags(String pattern, int start) {
		int count = 0;
		while ( start >= 0) {
			start = template.indexOf(pattern, start+1);
			if (start >= 0) {
				count++;
			}
		}
		return count;
	}

	protected String getQuery(String query, String ownerQuery) {
		String query1 = "";
		if (query.equals(ownerQuery)) {
			query1 = query;
		} else if (!ownerQuery.equals("") && query.startsWith(ownerQuery)) {
			query1 = query.substring(ownerQuery.length()+1);
		} else {
			String[] tokens = query.split("/");
			if (tokens.length >= 2) {
				query1 = tokens[tokens.length-2] + "/" + tokens[tokens.length-1];
			}
		}
		return query1;
	}	

	protected String getContextDataSection(MBDGElement element) {
		String data = "";
		int start = template.indexOf("${MBDG_QUERY_CONTEXT}");
		if (start == -1) {
			start = 0;
		}
		int open = template.indexOf("<data ", start);
		int close = template.indexOf("</data>", open);
		if (open >= 0 && close >= 0) {
			data =  template.substring(open, close);
		}
		return data;
	}
	
	protected String getFirstDataSection(StringBuilder content) {
		String data = "";
		int open = content.indexOf("<data ");
		int close = content.indexOf("</data>", open);
		if (open >= 0 && close >= 0) {
			data =  content.substring(open, close);
		}
		return data;
	}	
	
	protected String extractFromDataSection(String data, String openPattern, String closePattern) {
		String contextHandle = "";
		int start = data.indexOf(openPattern);
		if (start > 0) {
			int handleIndexOpen = start+openPattern.length();
			if (handleIndexOpen >= 0) {
				int handleIndexClose = data.indexOf(closePattern, handleIndexOpen);
				if (handleIndexClose > handleIndexOpen) {
					contextHandle = data.substring(handleIndexOpen, handleIndexClose);
				}
			}
		}
		return contextHandle;
	}	
}
