package apps;

import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;

import com.telelogic.rhapsody.core.IRPApplication;
import com.telelogic.rhapsody.core.IRPModelElement;

public class MBDGManager {
	private static final String CONFIGURATION_PATH = "${OMROOT}\\Profiles\\ModelBasedDocumentGeneration\\configuration.xml";
	MBDGVisitor visitor = null;
	
	private MBDGConfiguration configuration = new MBDGConfiguration();
	private IRPApplication rhapsody;
	
	public MBDGManager(IRPApplication rhapsody) {
		this.rhapsody = rhapsody;
		MBDGExpander.init(rhapsody);
		getConfiguration().load(CONFIGURATION_PATH);		
	}
	
	public void generateDocument(IRPModelElement selected, String path) throws IOException {
		getConfiguration().setDebugMode(false);
		updateConfiguration(path);
		if (deleteExistingDocument() == true) {
			generateDocument(selected, true);
		}
	}
	
	public void preview(IRPModelElement selected, Boolean withDescendants) throws IOException {
		SimpleDateFormat formatter = new SimpleDateFormat("yyyy_MM_dd_HH_mm_ss");
	    Date now = new Date();
	    String strDate = formatter.format(now);
		String path = getConfiguration().getTempDirectory()+File.separator+"Preview_"+strDate+".doc";
		getConfiguration().setDocumentPath(path);
		generateDocument(selected, withDescendants);
	}	
	
	public void generateDocument(IRPModelElement selected, boolean withDescendants) throws IOException {
    	if (visitor  == null) {
    		visitor = new MBDGVisitor(rhapsody, getConfiguration(), withDescendants);
    	} 
    	visitor.setWithDescendants(withDescendants);
    	visitor.visit(selected);
    }	

	public void updateConfiguration(String documentPath) {
    	MBDGConfiguration newConfiguration = getConfiguration().clone();
    	newConfiguration.setDocumentPath(documentPath);
    	if (!getConfiguration().equals(newConfiguration)) {
    		newConfiguration.save(CONFIGURATION_PATH);
    		setConfiguration(newConfiguration);
    	}
	}
	
	protected boolean deleteExistingDocument() {
		boolean res = true;
		String path = getConfiguration().getDocumentPath();
		File file = new File(path);
		if (file.exists()) {
			res = file.delete();
		}
		if (res == false) {
			MessageBox messageBox = new MessageBox(new Shell(), SWT.OK);
			messageBox.setMessage(String.format("The document [%s]\nis opened or locked on the disk.\nDocument generation will be canceled.", path));
			messageBox.setText("Generate Document");
			messageBox.open();
		}
		return res;
	}	

	public MBDGConfiguration getConfiguration() {
		return configuration;
	}

	public void setConfiguration(MBDGConfiguration configuration) {
		this.configuration = configuration;
	}
}
