package apps;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class MBDGMapExpander {
	private static final String PATTERN_PRFIX = "${MBDG_";
	private Map<String, List<String>> patterns;
	private Map<String, List<String>> templatePatterns;
	private Set<String> keywords;
	private StringBuilder template;

	public MBDGMapExpander(StringBuilder template) {
    	this.template = template;
		patterns = new HashMap<String, List<String>>();
		templatePatterns = new HashMap<String, List<String>>();
		keywords = new HashSet<String>();
		fillAvailableKeywords(template);
	}
	
	public void fillAvailableKeywords(StringBuilder template) {
		int start = 0;
		do {
			start = template.indexOf(PATTERN_PRFIX, start);
			if (start != -1) {
				int end = template.indexOf("}", start)+1;
				String pattern = template.substring(start, end);
				addKeyword(pattern);
				start = end;
			}
		} while (start >= 0);
	}
	
	public void addKeyword(String pattern) {
		if (!keywords.contains(pattern)) {
			keywords.add(pattern);
		}		
	}
	
	public void addPattern(String keyword, String childContent) {
		addPattern(keyword, new StringBuilder(childContent));
	}	

	public void addPattern(String keyword, StringBuilder childContent) {
		List<String> childContents = patterns.get(keyword);
		if (childContents == null) {
			childContents = new ArrayList<String>();
			patterns.put(keyword, childContents);
			addKeyword(keyword);
		}
		childContents.add(childContent.toString());
	}
	
	public void addTemplatePattern(String keyword, String childContent) {
		addTemplatePattern(keyword, new StringBuilder(childContent));
	}	

	public void addTemplatePattern(String keyword, StringBuilder childContent) {
		List<String> childContents = templatePatterns.get(keyword);
		if (childContents == null) {
			childContents = new ArrayList<String>();
			templatePatterns.put(keyword, childContents);			
		}
		childContents.add(childContent.toString());
	}	

	public void expand() {
	    MBDGExpander expander = new MBDGExpander(template);
		for (String keyword : keywords) {
			
			StringBuilder patternsContent = new StringBuilder();
			if (patterns.containsKey(keyword)) {
				for (String content : patterns.get(keyword)) {
					patternsContent.append(content);
				}
				expander.expand(keyword, patternsContent.toString());				
			}
			else if (templatePatterns.containsKey(keyword)) {
				StringBuilder templatePatternsContent = new StringBuilder();
				for (String content : templatePatterns.get(keyword)) {
					templatePatternsContent.append(content);
				}			
				expander.expandTemplateComment(keyword, templatePatternsContent.toString());
			} 
		}
	}
	
	public void removeTemplateComments() {
	    MBDGExpander expander = new MBDGExpander(template);
		for (String keyword : keywords) {
			if (!patterns.containsKey(keyword) && !templatePatterns.containsKey(keyword)) {
				expander.expandTemplateComment(keyword, "");
			}
		}
	}	

	public boolean containsKey(String keyword) {
		return keyword.contains(keyword);
	}
}
