package apps;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

public class NotificationDialog extends Dialog {
	
	public enum PreviewStatus { Preview, PreviewDescendants, None };
	
	private PreviewStatus status;
    private String message = "";
    private Shell shell;

    public NotificationDialog(Shell parent) {
        this(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
        shell = parent;
    }

    public NotificationDialog(Shell parent, int style) {
        super(parent, style);
        shell = parent;
    }

    public void open() {
        shell.setText(getText());
        createContents(shell);
        shell.pack();
        shell.open();
        Display display = getParent().getDisplay();
        while (!shell.isDisposed()) {
            if (!display.readAndDispatch()) {
                display.sleep();
            }
        }
    }

    private void createContents(final Shell shell) {
        shell.setLayout(new GridLayout(3, true));
        
        // Display the input box
        shell.setImage(shell.getDisplay().getSystemImage(SWT.ICON_QUESTION));        

        Label label = new Label(shell, SWT.NONE);
        label.setText("\n"+getMessage()+"\n\n");
        GridData data = new GridData(SWT.FILL, SWT.END, true, true);
        data.horizontalSpan = 3;
        label.setLayoutData(data);
        label.setAlignment(SWT.LEFT);        

        Button preview = new Button(shell, SWT.PUSH);
        preview.setText("Preview");
        data = new GridData(SWT.FILL, SWT.END, true, true);
        preview.setLayoutData(data);
        preview.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent event) {
                setStatus(PreviewStatus.Preview);
                shell.close();
            }
        });

        Button previewDescendants = new Button(shell, SWT.PUSH);
        previewDescendants.setText("Preview with Descendants");
        data = new GridData(SWT.FILL, SWT.END, true, true);
        previewDescendants.setLayoutData(data);
        previewDescendants.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent event) {
            	setStatus(PreviewStatus.PreviewDescendants);
            	shell.close();
            }
        });

        Button cancel = new Button(shell, SWT.PUSH);
        cancel.setText("Cancel");
        data = new GridData(SWT.FILL, SWT.END, true, true);
        cancel.setLayoutData(data);
        cancel.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent event) {
            	setStatus(PreviewStatus.None);
            	shell.close();
            }
        });

        shell.setDefaultButton(preview);
    }

    public static void main(String[] args) {
    	NotificationDialog dialog = new NotificationDialog(new Shell());
        dialog.setText("Title");
        dialog.setMessage("Message");

        dialog.open();
    }

	public PreviewStatus getStatus() {
		return status;
	}

	public void setStatus(PreviewStatus status) {
		this.status = status;
	}

	public String getMessage() {
		return message;
	}

	public void setMessage(String message) {
		this.message = message;
	}
}