package com.ibm.rhapsody.sysML;

import java.util.LinkedList;
import java.util.List;

import com.telelogic.rhapsody.core.IRPApplication;
import com.telelogic.rhapsody.core.IRPAssociationClass;
import com.telelogic.rhapsody.core.IRPAttribute;
import com.telelogic.rhapsody.core.IRPClassifier;
import com.telelogic.rhapsody.core.IRPCollection;
import com.telelogic.rhapsody.core.IRPDiagram;
import com.telelogic.rhapsody.core.IRPFlow;
import com.telelogic.rhapsody.core.IRPGraphEdge;
import com.telelogic.rhapsody.core.IRPGraphElement;
import com.telelogic.rhapsody.core.IRPInstance;
import com.telelogic.rhapsody.core.IRPLink;
import com.telelogic.rhapsody.core.IRPModelElement;
import com.telelogic.rhapsody.core.IRPPackage;
import com.telelogic.rhapsody.core.IRPPort;
import com.telelogic.rhapsody.core.IRPSysMLPort;
import com.telelogic.rhapsody.core.IRPTag;
import com.telelogic.rhapsody.core.RhapsodyAppServer;

public class CreateUpdateDirectionFlows {
	private IRPApplication rhapsody = null;
	
	public CreateUpdateDirectionFlows(IRPApplication rhapsody) {
		this.rhapsody = rhapsody;
	}
	
	public void CreateUpdateFlows() {
		if(rhapsody == null)
			return;
		
		IRPGraphEdge selectedGraphLink = null;
		IRPGraphElement selectedGraphElem = RhpUtils.getSelectedGraphicalElement(null);
		if(selectedGraphElem instanceof IRPGraphEdge)
			selectedGraphLink = (IRPGraphEdge)selectedGraphElem;
		IRPModelElement selectedElem = rhapsody.getSelectedElement();
		if(selectedGraphLink != null && selectedElem != null && selectedElem instanceof IRPLink) {
			IRPDiagram diagram = selectedGraphElem.getDiagram();
			IRPLink link = (IRPLink)selectedElem;
			IRPSysMLPort fromFlowPort = link.getFromSysMLPort();
			IRPSysMLPort toFlowPort = link.getToSysMLPort();
			IRPPort fromPort = link.getFromPort();
			IRPPort toPort = link.getToPort();
			IRPInstance fromPart = link.getFrom();
			IRPInstance toPart = link.getTo();
			IRPClassifier fromPortType = null;
			IRPClassifier toPortType = null;
			String fromPortName = null;
			String toPortName = null;
			if(fromFlowPort != null && toFlowPort != null) {
				fromPortType = fromFlowPort.getOtherClass();
				toPortType = toFlowPort.getOtherClass();
				fromPortName = fromFlowPort.getName();
				toPortName = toFlowPort.getName();
			}
			else if(fromPort != null && toPort != null) {
				fromPortType = fromPort.getOtherClass();
				toPortType = toPort.getOtherClass();
				fromPortName = fromPort.getName();
				toPortName = toPort.getName();
			}
				
			if(fromPortType != null && toPortType != null) {
				int bFromPortReversed = 0;
				int bToPortReversed = 0;
				if(fromPort != null && toPort != null)
				{
					bFromPortReversed = fromPort.getIsReversed();
					bToPortReversed = toPort.getIsReversed();
				}
				else if(fromFlowPort != null && toFlowPort != null) {
					bFromPortReversed = fromFlowPort.getIsReversed();
					bToPortReversed = toFlowPort.getIsReversed();
				}
				
				List<IRPAttribute> fromPortInFlowProperties = new LinkedList<IRPAttribute>();
				RhpUtils.collectFlowProperties(fromPortType, true, bFromPortReversed, fromPortInFlowProperties);
					
				List<IRPAttribute> fromPortOutFlowProperties = new LinkedList<IRPAttribute>();
				RhpUtils.collectFlowProperties(fromPortType, false, bFromPortReversed, fromPortOutFlowProperties);
					
				List<IRPAttribute> toPortInFlowProperties = new LinkedList<IRPAttribute>();
				RhpUtils.collectFlowProperties(toPortType, true, bToPortReversed, toPortInFlowProperties);
					
				List<IRPAttribute> toPortOutFlowProperties = new LinkedList<IRPAttribute>();
				RhpUtils.collectFlowProperties(toPortType, false, bToPortReversed, toPortOutFlowProperties);
					
				IRPPackage pkg = RhpUtils.getPackageOfBlock(fromPortType);
				List<IRPAttribute> matchingInFlowProp = findMatchingFlowProperties(fromPortInFlowProperties, toPortOutFlowProperties);
				if(pkg != null && matchingInFlowProp != null && matchingInFlowProp.size() > 0) {
					String flowName = fromPortName + "_" + toPortName + "_In";
					IRPFlow flow = RhpUtils.findFlow(pkg, flowName);
					if(flow == null)
						flow = pkg.addFlows(flowName);
					if(flow != null) {
						if(fromPort != null && toPort != null) {
							flow.setEnd1ViaPort(toPart, toPort);
							flow.setEnd2ViaPort(fromPart, fromPort);
						}
						else if(fromFlowPort != null && toFlowPort != null){
							flow.setEnd1ViaSysMLPort(toPart, toFlowPort);
							flow.setEnd2ViaSysMLPort(fromPart, fromFlowPort);
						}
						
						IRPTag tag = flow.getTag("realization");
						if(tag == null) {
							tag = (IRPTag)flow.addNewAggr("Tag", "realization");
							tag.addElementDefaultValue(link);
						}
						else {
							//first, remove old conveyeds
							IRPCollection conveyeds = flow.getConveyed();
							if(conveyeds != null) {
								for(int i = 1 ; i <= conveyeds.getCount() ; i++) {
									IRPModelElement conveyed = (IRPModelElement) conveyeds.getItem(i);
									flow.removeConveyed(conveyed);
								}
							}
						}
							
						for(IRPAttribute attr : matchingInFlowProp) {
							flow.addConveyed(attr);
						}
					}
				}
			
				List<IRPAttribute> matchingOutFlowProp = findMatchingFlowProperties(fromPortOutFlowProperties, toPortInFlowProperties);
				if(pkg != null && matchingOutFlowProp != null && matchingOutFlowProp.size() > 0) {
					String flowName = fromPortName + "_" + toPortName + "_Out";
					IRPFlow flow = RhpUtils.findFlow(pkg, flowName);
					if(flow == null)
						flow = pkg.addFlows(flowName);
					if(flow != null) {
						if(fromPort != null && toPort != null) {
							flow.setEnd1ViaPort(fromPart, fromPort);
							flow.setEnd2ViaPort(toPart, toPort);
						}
						else if(fromFlowPort != null && toFlowPort != null) {
							flow.setEnd1ViaSysMLPort(fromPart, fromFlowPort);
							flow.setEnd2ViaSysMLPort(toPart, toFlowPort);
						}
						
						IRPTag tag = flow.getTag("realization");
						if(tag == null) {
							tag = (IRPTag)flow.addNewAggr("Tag", "realization");
							tag.addElementDefaultValue(link);
						}
						else {
							//first, remove old conveyeds
							IRPCollection conveyeds = flow.getConveyed();
							if(conveyeds != null) {
								for(int i = 1 ; i <= conveyeds.getCount() ; i++) {
									IRPModelElement conveyed = (IRPModelElement) conveyeds.getItem(i);
									flow.removeConveyed(conveyed);
								}
							}
						}
						
						for(IRPAttribute attr : matchingOutFlowProp) {
							flow.addConveyed(attr);
						}
					}
				}
					
				if(diagram != null) {
					IRPCollection graphElements = rhapsody.createNewCollection();
					IRPGraphElement srcNode = selectedGraphLink.getSource();
					IRPGraphElement trgNode = selectedGraphLink.getTarget();
					if(srcNode != null && trgNode != null) {
						graphElements.addGraphicalItem(srcNode);
						graphElements.addGraphicalItem(trgNode);
						diagram.completeRelations(graphElements, 0);
						rhapsody.refreshAllViews();
					}	
				}
			} 
		}
	}
	
	private List<IRPAttribute> findMatchingFlowProperties(
			List<IRPAttribute> fromFlowProperties,
			List<IRPAttribute> toFlowProperties) {
		List<IRPAttribute> matchingFlowProp = null;
		if(fromFlowProperties != null && fromFlowProperties.size() > 0) {
			for(IRPAttribute fromFlowProp : fromFlowProperties) {
				for(IRPAttribute toFlowProp : toFlowProperties) {
					boolean bMatch = false;
					if(fromFlowProp.equals(toFlowProp))
						bMatch = true;
					else if(fromFlowProp.getName().equals(toFlowProp.getName())) {
						IRPClassifier fromType = fromFlowProp.getType();
						IRPClassifier toType = toFlowProp.getType();
						if(fromType != null && toType != null && fromType.equals(toType))
							bMatch = true;
					}
					else {
						//check if this FlowProperty is explicitly mapped
						IRPCollection references = fromFlowProp.getReferences();
						if(references != null && references.getCount() > 0) {
							for(int i = 1 ; i <= references.getCount() ; i++) {
								IRPModelElement elem = (IRPModelElement) references.getItem(i);
								if(elem instanceof IRPFlow && RhpUtils.getStereotype(elem, "BindingConnector") != null) {
									IRPFlow flow = (IRPFlow)elem;
									IRPModelElement owner = flow.getOwner();
									if(owner != null && owner instanceof IRPAssociationClass) {
										IRPModelElement end1 = flow.getEnd1();
										IRPModelElement end2 = flow.getEnd2();
										if(end1 != null && end1.equals(fromFlowProp) && 
											end2 != null && end2.equals(toFlowProp)) {
											bMatch = true;
											break;
										}
										else if(end2 != null && end2.equals(fromFlowProp) && 
											end1 != null && end1.equals(toFlowProp)) {
											bMatch = true;
											break;
										}
									}
								}
							}
						}
					}
					
					if(bMatch) {
						if(matchingFlowProp == null)
							matchingFlowProp = new LinkedList<IRPAttribute>();
						matchingFlowProp.add(fromFlowProp);
						break;
					}
				}
			}
		}
		return matchingFlowProp;
	}

	public static void main(String args[]) {
		IRPApplication rhapsody = RhapsodyAppServer.getActiveRhapsodyApplication();
		CreateUpdateDirectionFlows createFlows = new CreateUpdateDirectionFlows(rhapsody);
		createFlows.CreateUpdateFlows();
	}
}
