package com.ibm.rhapsody.sysML;

import com.telelogic.rhapsody.core.IRPApplication;
import com.telelogic.rhapsody.core.IRPArgument;
import com.telelogic.rhapsody.core.IRPAttribute;
import com.telelogic.rhapsody.core.IRPClass;
import com.telelogic.rhapsody.core.IRPClassifier;
import com.telelogic.rhapsody.core.IRPCollection;
import com.telelogic.rhapsody.core.IRPEvent;
import com.telelogic.rhapsody.core.IRPEventReception;
import com.telelogic.rhapsody.core.IRPInstanceValue;
import com.telelogic.rhapsody.core.IRPInterfaceItem;
import com.telelogic.rhapsody.core.IRPModelElement;
import com.telelogic.rhapsody.core.IRPPackage;
import com.telelogic.rhapsody.core.IRPStereotype;
import com.telelogic.rhapsody.core.IRPTag;
import com.telelogic.rhapsody.core.IRPValueSpecification;
import com.telelogic.rhapsody.core.RPUserPlugin;



public class SysMLPlugin extends RPUserPlugin {

	private IRPApplication mRhpApp = null;
	
	@Override
	public void OnMenuItemSelect(String menuItem) {
		try {
			if(menuItem.equals("SysML\\Update ChangeEvent Arguments")) {
				UpdateChangeEventArguments();
			}
			else if(menuItem.equals("SysML\\Create/Update Direction Flows")) {
				CreateUpdateDirectionFlows createFlows = new CreateUpdateDirectionFlows(mRhpApp);
				createFlows.CreateUpdateFlows();
			}
			else if(menuItem.equals("SysML\\Create Swimlanes From Parts")) {
				CreateSwimlaneFromParts createSwimlaneFromParts = new CreateSwimlaneFromParts(mRhpApp);
				createSwimlaneFromParts.createSwimlanes();
			}
			else {
				mRhpApp.writeToOutputWindow("Log", "unknown command - " + menuItem);
			}
		} catch (Exception e) {
			mRhpApp.writeToOutputWindow("Log", e.getMessage());
		}
	}

	private void UpdateChangeEventArguments() {
		if(mRhpApp != null) {
			IRPModelElement elem = mRhpApp.getSelectedElement();
			if(elem instanceof IRPEvent) {
				IRPEvent event = (IRPEvent)elem;
				UpdateEventArguments(event);
			}
			else if(elem instanceof IRPPackage) {
				IRPPackage pkg = (IRPPackage)elem;
				IRPCollection col = pkg.getEvents();
				for (int i = 1 ; i <= col.getCount() ; i++) {
					IRPEvent event = (IRPEvent)col.getItem(i);
					UpdateEventArguments(event);
				}
			}
		}
	}

	private void UpdateEventArguments(IRPEvent event) {
		if(event != null) {
			IRPStereotype stereotype = event.getStereotype();
			if((stereotype != null) && (stereotype.getIsNewTerm() == 1) && 
					stereotype.getName().equals("changeStructuralFeatureEvent")) {
				IRPTag tag = event.getTag("structuralFeature");
				if(tag != null) {
					IRPAttribute attr = null;
					IRPCollection col = tag.getValueSpecifications();
					if(col != null) {
						for(int i = 1 ; i <= col.getCount() ; i++) {
							IRPValueSpecification valueSpec = (IRPValueSpecification) col.getItem(i);
							if(valueSpec instanceof IRPInstanceValue) {
								IRPInstanceValue instVal = (IRPInstanceValue)valueSpec;
								if(instVal != null) {
									IRPModelElement value = instVal.getValue();
									if(value instanceof IRPAttribute) {
										attr = (IRPAttribute)value;
										break;
									}
								}
							}
						}
					}
					if(attr != null) {
						IRPClassifier attrType = attr.getType();
						
						//first delete all arguments of the event
						IRPCollection args = event.getArguments();
						for(int i = 1 ; i <= args.getCount() ; i++) {
							IRPArgument arg = (IRPArgument)args.getItem(i);
							arg.deleteFromProject();
						}
						
						//next create 2 new argument 'oldValue', and 'newValue'
						IRPArgument arg = event.addArgument("oldValue");
						if(arg != null) {
							arg.setType(attrType);
						}
						arg = event.addArgument("newValue");
						if(arg != null) {
							arg.setType(attrType);
						}
						
						//last, create an event reception in the class owner of the attribute,
						//according to the event
						IRPModelElement elem = attr.getOwner();
						if(elem instanceof IRPClass) {
							IRPClass clsOwner = (IRPClass)elem;
							IRPInterfaceItem item = clsOwner.findInterfaceItem(event.getSignatureNoArgNames());
							if(item == null) {
								IRPEventReception recep = clsOwner.addReception(event.getName());
								if(recep != null) {
									recep.setEvent(event);
								}
							}
						}
					}
				}
			}
		}
	}

	@Override
	public void OnTrigger(String trigger) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public boolean RhpPluginCleanup() {
		
		return false;
	}

	@Override
	public void RhpPluginFinalCleanup() {
		mRhpApp = null;
	}

	@Override
	public void RhpPluginInit(IRPApplication rpyApplication) {
		// TODO Auto-generated method stub
		mRhpApp = rpyApplication;
	}

	@Override
	public void RhpPluginInvokeItem() {
		// TODO Auto-generated method stub
		
	}

}
